<?php

namespace Drupal\edit_plus_sweet\EventSubscriber\LayoutBuilderBlockForm;

use Drupal\edit_plus_sweet\Events\LayoutBuilderBlockFormAlterEvent;
use Drupal\Core\Template\Attribute;

/**
 * Alters the block form of the 'heading' custom block bundle.
 */
class HeadingCustomizations extends EnhancedTitleBase {

  /**
   * {@inheritdoc}
   */
  protected static array $configurableProperties = [
    'first_color',
    'second_color',
    'gradient_rotation',
    'alignment',
  ];

  /**
   * {@inheritdoc}
   */
  public $blockId = 'heading';

  /**
   * {@inheritdoc}
   */
  public function formAlter(LayoutBuilderBlockFormAlterEvent $event) {
    parent::formAlter($event);
    $form = $event->getForm();
    // Callouts need the block's label, so we're going to remove the ability to
    // disable it from the form.
    $form['settings']['label']['#access'] = TRUE;
    $form['settings']['label_display']['#value'] = TRUE;
    $form['settings']['label_display']['#access'] = FALSE;
    $form['settings']['label_display']['#default_value'] = TRUE;
    $event->setForm($form);
  }

  /**
   * {@inheritdoc}
   */
  public static function addTitleOptions(array &$form, array $config) {
    $form['alignment'] = [
      '#title' => t('Heading Alignment'),
      '#type' => 'select',
      '#options' => [
        'text-align-left' => t('Left'),
        'text-align-center' => t('Center'),
        'text-align-right' => t('Right'),
      ],
      '#required' => TRUE,
      '#default_value' => $config['alignment'] ?? 'text-align-left',
    ];
    parent::addTitleOptions($form, $config);
  }

  /**
   * {@inheritdoc}
   */
  public static function setTitleAttributes(Attribute $title_attributes, array $config) {
    parent::setTitleAttributes($title_attributes, $config);
    $title_attributes->addClass('heading-formatter');
    if (!empty($config['alignment'])) {
      $title_attributes->addClass($config['alignment']);
    }
  }

}
