<?php

namespace Drupal\edit_plus_sweet\EventSubscriber\LayoutBuilderBlockForm;

use Drupal\layout_builder\Form\AddBlockForm;
use Drupal\layout_builder\Form\UpdateBlockForm;
use Drupal\layout_builder\Plugin\Block\InlineBlock;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\edit_plus_sweet\Events\LayoutBuilderBlockFormAlterEvent;

abstract class LayoutBuilderBlockFormBase implements LayoutBuilderBlockFormInterface, EventSubscriberInterface {

  /**
   * Required Block ID to act on.
   *
   * @var string
   */
  public $blockId;

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events = [];
    $events[LayoutBuilderBlockFormAlterEvent::class][] = ['onBlockFormAlter', 10];
    return $events;
  }

  /**
   * Checks the Block ID and calls formAlter().
   *
   * @param \Drupal\edit_plus_sweet\Events\LayoutBuilderBlockFormAlterEvent $event
   *   The form alteration event.
   *
   * @throws \ReflectionException
   */
  public function onBlockFormAlter(LayoutBuilderBlockFormAlterEvent $event) {
    if (!$this->blockId) {
      return;
    }
    $formObject = $event->getFormState()->getFormObject();
    $block = FALSE;
    if ($formObject instanceof AddBlockForm || $formObject instanceof UpdateBlockForm) {
      $p = new \ReflectionProperty($formObject, 'block');
      $p->setAccessible(TRUE);
      /** @var $block InlineBlock **/
      // This isn't actually an inline block all the time, but the interface is
      // standard, and we only operate on inline call to action blocks.
      $block = $p->getValue($formObject);
    }
    if (!$block || ($block->getPluginId() !== "inline_block:{$this->blockId}" && $block->getPluginId() !== $this->blockId)) {
      return;
    }

    $event->setBlock($block);
    $this->formAlter($event);
  }

}
