<?php

namespace Drupal\edit_plus_sweet\Events;

use Drupal\Core\Block\BlockPluginInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\layout_builder\Plugin\Block\InlineBlock;
use Drupal\layout_builder\SectionComponent;
use Drupal\layout_builder\SectionStorageInterface;
use Symfony\Contracts\EventDispatcher\Event;

class LayoutBuilderBlockFormAlterEvent extends Event {

  /**
   * The form array being altered.
   *
   * @var array
   */
  protected $form;

  /**
   * The formState object for the form being altered.
   *
   * @var FormStateInterface
   */
  protected $formState;

  /**
   * The layout_builder instance storage the form appears within.
   *
   * @var SectionStorageInterface
   */
  protected $storage;

  /**
   * The region in which the block does or will appear.
   *
   * @var string
   */
  protected $region;

  /**
   * The block's configuration.
   *
   * This may be empty in the case that the block is being configured for the
   * first time.
   *
   * @var array
   */
  protected $configuration;

  /**
   * The section component if available.
   *
   * @var SectionComponent|null
   */
  protected $component;

  /**
   * The inline block.
   *
   * @var
   */
  protected $block;

  /**
   * LayoutBuilderBlockFormAlterEvent constructor.
   * @param array $form
   * @param FormStateInterface $formState
   * @param SectionStorageInterface $storage
   * @param string $region
   * @param array $configuration
   * @param SectionComponent|null $param
   */
  public function __construct(array $form, FormStateInterface $formState, SectionStorageInterface $storage, string $region, array $configuration, SectionComponent $component = NULL) {
    $this->form = $form;
    $this->formState = $formState;
    $this->storage = $storage;
    $this->region = $region;
    $this->configuration = $configuration;
    $this->component = $component;
  }

  /**
   * @return array
   */
  public function getForm(): array {
    return $this->form;
  }

  /**
   * @param array $form
   */
  public function setForm(array $form): void {
    $this->form = $form;
  }

  /**
   * @return FormStateInterface
   */
  public function getFormState(): FormStateInterface {
    return $this->formState;
  }

  /**
   * @return SectionStorageInterface
   */
  public function getStorage(): SectionStorageInterface {
    return $this->storage;
  }

  /**
   * @return string
   */
  public function getRegion(): string {
    return $this->region;
  }

  /**
   * @return array
   */
  public function getConfiguration(): array {
    return $this->configuration;
  }

  /**
   * @return SectionComponent|null
   */
  public function getComponent() {
    return $this->component;
  }

  /**
   * @return \Drupal\Core\Block\BlockPluginInterface
   */
  public function getBlock(): BlockPluginInterface {
    return $this->block;
  }

  /**
   * @param \Drupal\Core\Block\BlockPluginInterface $block
   */
  public function setBlock(BlockPluginInterface $block): void {
    $this->block = $block;
  }

}
